@@ -23,12 +23,6 @@
 
 #include "engines/mohawk/archive.h"
 
-inline uint32 SWAP_BYTES_32(uint32 a) {
-	const uint16 low = (uint16)a, high = (uint16)(a >> 16);
-	return ((uint32)(uint16)((low >> 8) | (low << 8)) << 16)
-		   | (uint16)((high >> 8) | (high << 8));
-}
-
 Common::String MohawkArchive::tag2string(uint32 tag) {
 	char str[5];
 	str[0] = (char)(tag >> 24);
@@ -54,7 +48,7 @@ MohawkArchive::MohawkArchive() {
 }
 
 void MohawkArchive::close() {
-	delete _mhk; _mhk = NULL;
+	_mhk = NULL;
 	delete[] _types; _types = NULL;
 	delete[] _fileTable; _fileTable = NULL;
 
@@ -63,7 +57,7 @@ void MohawkArchive::close() {
 	_curExTypeIndex = 0;
 }
 
-void MohawkArchive::open(Common::SeekableReadStream *stream) {
+void MohawkArchive::open(Common::File *stream) {
 	// Make sure no other file is open...
 	close();
 	_mhk = stream;
@@ -94,7 +88,7 @@ void MohawkArchive::open(Common::SeekableReadStream *stream) {
 	/////////////////////////////////
 
 	// Type Table
-	_mhk->seek(_rsrc.abs_offset);
+	_mhk->seek(_rsrc.abs_offset, SEEK_SET);
 	_typeTable.name_offset = _mhk->readUint16BE();
 	_typeTable.resource_types = _mhk->readUint16BE();
 
@@ -114,7 +108,7 @@ void MohawkArchive::open(Common::SeekableReadStream *stream) {
 			debug (3, "Type[%02d]: Tag = \'%s\' ResTable Offset = %04x  NameTable Offset = %04x", i, tag2str(_types[i].tag), _types[i].resource_table_offset, _types[i].name_table_offset);
 
 		//Resource Table
-		_mhk->seek(_rsrc.abs_offset + _types[i].resource_table_offset);
+		_mhk->seek(_rsrc.abs_offset + _types[i].resource_table_offset, SEEK_SET);
 		_types[i].resTable.resources = _mhk->readUint16BE();
 
 		debug (3, "Resources = %04x", _types[i].resTable.resources);
@@ -129,7 +123,7 @@ void MohawkArchive::open(Common::SeekableReadStream *stream) {
 		}
 
 		// Name Table
-		_mhk->seek(_rsrc.abs_offset + _types[i].name_table_offset);
+		_mhk->seek(_rsrc.abs_offset + _types[i].name_table_offset, SEEK_SET);
 		_types[i].nameTable.num = _mhk->readUint16BE();
 
 		debug (3, "Names = %04x", _types[i].nameTable.num);
@@ -144,7 +138,7 @@ void MohawkArchive::open(Common::SeekableReadStream *stream) {
 
 			// Name List
 			uint32 pos = _mhk->pos();
-			_mhk->seek(_rsrc.abs_offset + _typeTable.name_offset + _types[i].nameTable.entries[j].offset);
+			_mhk->seek(_rsrc.abs_offset + _typeTable.name_offset + _types[i].nameTable.entries[j].offset, SEEK_SET);
 			char c = (char)_mhk->readByte();
 			while (c != 0) {
 				_types[i].nameTable.entries[j].name += c;
@@ -154,16 +148,16 @@ void MohawkArchive::open(Common::SeekableReadStream *stream) {
 			debug (3, "Name = \'%s\'", _types[i].nameTable.entries[j].name.c_str());
 
 			// Get back to next entry
-			_mhk->seek(pos);
+			_mhk->seek(pos, SEEK_SET);
 		}
 
 		// Return to next TypeTable entry
-		_mhk->seek(_rsrc.abs_offset + (i + 1) * 8 + 4);
+		_mhk->seek(_rsrc.abs_offset + (i + 1) * 8 + 4, SEEK_SET);
 
 		debug (3, "\n");
 	}
 
-	_mhk->seek(_rsrc.abs_offset + _rsrc.file_table_offset);
+	_mhk->seek(_rsrc.abs_offset + _rsrc.file_table_offset, SEEK_SET);
 	_fileTableAmount = _mhk->readUint32BE();
 	_fileTable = new FileTable[_fileTableAmount];
 
@@ -182,20 +176,8 @@ void MohawkArchive::open(Common::SeekableReadStream *stream) {
 	}
 }
 
-bool MohawkArchive::hasResource(uint32 tag, uint16 id) {
-	if (!_mhk)
-		return false;
-
-	int16 typeIndex = getTypeIndex(tag);
-
-	if (typeIndex < 0)
-		return false;
-
-	return (getIdIndex(typeIndex, id) >= 0);
-}
-
 MohawkOutputStream MohawkArchive::getRawData(uint32 tag, uint16 id) {
-	MohawkOutputStream output = { 0, 0, 0, 0, 0, "" };
+	MohawkOutputStream output = { 0, 0, 0, 0, 0, 0, 0, "" };
 
 	if (!_mhk)
 		return output;
@@ -215,20 +197,32 @@ MohawkOutputStream MohawkArchive::getRawData(uint32 tag, uint16 id) {
 
 	// WORKAROUND: tMOV resources pretty much ignore the size part of the file table,
 	// as the original just passed the full Mohawk file to QuickTime and the offset.
+	// We set the resource size to the number of bytes till the beginning of the next
+	// resource in the archive.
 	// We need to do this because of the way Mohawk is set up (this is much more "proper"
-	// than passing _mhk at the right offset). We may want to do that in the future, though.
-	if (_types[typeIndex].tag == ID_TMOV) {
-		if (fileTableIndex == _fileTableAmount - 1)
-			output.stream = new Common::SeekableSubReadStream(_mhk, _fileTable[fileTableIndex].offset, _mhk->size());
-		else
-			output.stream = new Common::SeekableSubReadStream(_mhk, _fileTable[fileTableIndex].offset, _fileTable[fileTableIndex + 1].offset);
+	// than passing _stream at the right offset). We may want to do that in the future, though.
+	if (_types[_curExType].tag == ID_TMOV) {
+		uint16 nextFileIndex = fileTableIndex + 1;
+		output.size = 0;
+		while (output.size == 0) {
+			if (nextFileIndex == _fileTableAmount)
+				output.size = _mhk->size() - _fileTable[fileTableIndex].offset;
+			else
+				output.size = _fileTable[nextFileIndex].offset - _fileTable[fileTableIndex].offset;
+
+			// Loop because two entries in the file table may point to the same data
+			// in the archive.
+			nextFileIndex++;
+		}
 	} else
-		output.stream = new Common::SeekableSubReadStream(_mhk, _fileTable[fileTableIndex].offset, _fileTable[fileTableIndex].offset + _fileTable[fileTableIndex].dataSize);
+		output.size = _fileTable[fileTableIndex].dataSize;
 
 	output.tag = tag;
 	output.id = id;
 	output.index = fileTableIndex;
 	output.flags = _fileTable[fileTableIndex].flags;
+	output.offset = _fileTable[fileTableIndex].offset;
+
 	for (uint16 i = 0; i < _types[typeIndex].nameTable.num; i++) {
 		if (_types[typeIndex].nameTable.entries[i].index == fileTableIndex+1) {
 			output.name = _types[typeIndex].nameTable.entries[i].name;
@@ -236,11 +230,14 @@ MohawkOutputStream MohawkArchive::getRawData(uint32 tag, uint16 id) {
 		}
 	}
 
+	output.stream = _mhk;
+	output.stream->seek(output.offset, SEEK_SET);
+
 	return output;
 }
 
 MohawkOutputStream MohawkArchive::getNextFile() {
-	MohawkOutputStream output = { 0, 0, 0, 0, 0, "" };
+	MohawkOutputStream output = { 0, 0, 0, 0, 0, 0, 0, "" };
 
 	if (_curExType >= _typeTable.resource_types) // No more!
 		return output;
@@ -255,21 +252,35 @@ MohawkOutputStream MohawkArchive::getNextFile() {
 
 	uint16 fileTableIndex = _types[_curExType].resTable.entries[_curExTypeIndex].index - 1;
 
-	// For some unknown reason, all tMOV resources have incorrect sizes. We correct this by getting the differences between offsets.
-	uint32 dataSize = 0;
+	// WORKAROUND: tMOV resources pretty much ignore the size part of the file table,
+	// as the original just passed the full Mohawk file to QuickTime and the offset.
+	// We set the resource size to the number of bytes till the beginning of the next
+	// resource in the archive.
+	// We need to do this because of the way Mohawk is set up (this is much more "proper"
+	// than passing _stream at the right offset). We may want to do that in the future, though.
 	if (_types[_curExType].tag == ID_TMOV) {
-		if (fileTableIndex == _fileTableAmount - 1)
-			dataSize = _mhk->size() - _fileTable[fileTableIndex].offset;
-		else
-			dataSize = _fileTable[fileTableIndex + 1].offset - _fileTable[fileTableIndex].offset;
+		uint16 nextFileIndex = fileTableIndex + 1;
+		output.size = 0;
+		while (output.size == 0) {
+			if (nextFileIndex == _fileTableAmount)
+				output.size = _mhk->size() - _fileTable[fileTableIndex].offset;
+			else
+				output.size = _fileTable[nextFileIndex].offset - _fileTable[fileTableIndex].offset;
+
+			// Loop because two entries in the file table may point to the same data
+			// in the archive.
+			nextFileIndex++;
+		}
 	} else
-		dataSize = _fileTable[fileTableIndex].dataSize;
+		output.size = _fileTable[fileTableIndex].dataSize;
 
-	output.stream = new Common::SeekableSubReadStream(_mhk, _fileTable[fileTableIndex].offset, _fileTable[fileTableIndex].offset + dataSize, false);
 	output.tag = _types[_curExType].tag;
 	output.id = _types[_curExType].resTable.entries[_curExTypeIndex].id;
 	output.index = fileTableIndex;
 	output.flags = _fileTable[fileTableIndex].flags;
+	output.offset = _fileTable[fileTableIndex].offset;
+	output.stream = _mhk;
+	output.stream->seek(output.offset, SEEK_SET);
 
 	for (uint16 i = 0; i < _types[_curExType].nameTable.num; i++) {
 		if (_types[_curExType].nameTable.entries[i].index == fileTableIndex+1) {
@@ -282,7 +293,7 @@ MohawkOutputStream MohawkArchive::getNextFile() {
 	return output;
 }
 
-void LivingBooksArchive_v1::open(Common::SeekableReadStream *stream) {
+void LivingBooksArchive_v1::open(Common::File *stream) {
 	close();
 	_mhk = stream;
 
@@ -313,7 +324,7 @@ void LivingBooksArchive_v1::open(Common::SeekableReadStream *stream) {
 			uint32 oldPos = _mhk->pos();
 
 			// Resource Table/File Table
-			_mhk->seek(_types[i].resource_table_offset);
+			_mhk->seek(_types[i].resource_table_offset, SEEK_SET);
 			_types[i].resTable.resources = _mhk->readUint16BE();
 			_types[i].resTable.entries = new OldType::ResourceTable::Entries[_types[i].resTable.resources];
 
@@ -322,15 +333,15 @@ void LivingBooksArchive_v1::open(Common::SeekableReadStream *stream) {
 				_types[i].resTable.entries[j].offset = _mhk->readUint32BE();
 				_types[i].resTable.entries[j].size = _mhk->readByte() << 16;
 				_types[i].resTable.entries[j].size += _mhk->readUint16BE();
-				_mhk->skip(5); // Unknown (always 0?)
+				_mhk->seek(5, SEEK_CUR); // Unknown (always 0?)
 
 				debug (4, "Entry[%02x]: ID = %04x (%d)\tOffset = %08x, Size = %08x", j, _types[i].resTable.entries[j].id, _types[i].resTable.entries[j].id, _types[i].resTable.entries[j].offset, _types[i].resTable.entries[j].size);
 			}
 
-			_mhk->seek(oldPos);
+			_mhk->seek(oldPos, SEEK_SET);
 			debug (3, "\n");
 		}
-	} else if (SWAP_BYTES_32(headerSize) == 6) { // We're in Little Endian mode (Windows)
+	} else if (SWAP_32(headerSize) == 6) { // We're in Little Endian mode (Windows)
 		_mhk->readUint16LE(); // Resource Table Size
 		_typeTable.resource_types = _mhk->readUint16LE();
 		_types = new OldType[_typeTable.resource_types];
@@ -347,7 +358,7 @@ void LivingBooksArchive_v1::open(Common::SeekableReadStream *stream) {
 			uint32 oldPos = _mhk->pos();
 
 			// Resource Table/File Table
-			_mhk->seek(_types[i].resource_table_offset);
+			_mhk->seek(_types[i].resource_table_offset, SEEK_SET);
 			_types[i].resTable.resources = _mhk->readUint16LE();
 			_types[i].resTable.entries = new OldType::ResourceTable::Entries[_types[i].resTable.resources];
 
@@ -360,7 +371,7 @@ void LivingBooksArchive_v1::open(Common::SeekableReadStream *stream) {
 				debug (4, "Entry[%02x]: ID = %04x (%d)\tOffset = %08x, Size = %08x", j, _types[i].resTable.entries[j].id, _types[i].resTable.entries[j].id, _types[i].resTable.entries[j].offset, _types[i].resTable.entries[j].size);
 			}
 
-			_mhk->seek(oldPos);
+			_mhk->seek(oldPos, SEEK_SET);
 			debug (3, "\n");
 		}
 	} else
@@ -369,7 +380,7 @@ void LivingBooksArchive_v1::open(Common::SeekableReadStream *stream) {
 }
 
 MohawkOutputStream LivingBooksArchive_v1::getRawData(uint32 tag, uint16 id) {
-	MohawkOutputStream output = { 0, 0, 0, 0, 0, "" };
+	MohawkOutputStream output = { 0, 0, 0, 0, 0, 0, 0, "" };
 
 	if (!_mhk)
 		return output;
@@ -384,16 +395,19 @@ MohawkOutputStream LivingBooksArchive_v1::getRawData(uint32 tag, uint16 id) {
 	if (idIndex < 0)
 		return output;
 
-	output.stream = new Common::SeekableSubReadStream(_mhk, _types[typeIndex].resTable.entries[idIndex].offset, _types[typeIndex].resTable.entries[idIndex].offset + _types[typeIndex].resTable.entries[idIndex].size);
+	output.offset = _types[typeIndex].resTable.entries[idIndex].offset;
+	output.size   = _types[typeIndex].resTable.entries[idIndex].size;
 	output.tag = tag;
 	output.id = id;
 	output.index = idIndex;
+	output.stream = _mhk;
+	output.stream->seek(output.offset, SEEK_SET);
 
 	return output;
 }
 
 MohawkOutputStream LivingBooksArchive_v1::getNextFile() {
-	MohawkOutputStream output = { 0, 0, 0, 0, 0, "" };
+	MohawkOutputStream output = { 0, 0, 0, 0, 0, 0, 0, "" };
 
 	if (_curExType >= _typeTable.resource_types) // No more!
 		return output;
@@ -406,16 +420,19 @@ MohawkOutputStream LivingBooksArchive_v1::getNextFile() {
 			return output;
 	}
 
-	output.stream = new Common::SeekableSubReadStream(_mhk, _types[_curExType].resTable.entries[_curExTypeIndex].offset, _types[_curExType].resTable.entries[_curExTypeIndex].offset + _types[_curExType].resTable.entries[_curExTypeIndex].size);
+	output.offset = _types[_curExType].resTable.entries[_curExTypeIndex].offset;
+	output.size   = _types[_curExType].resTable.entries[_curExTypeIndex].size;
 	output.tag = _types[_curExType].tag;
 	output.id = _types[_curExType].resTable.entries[_curExTypeIndex].id;
 	output.index = _curExType;
+	output.stream = _mhk;
+	output.stream->seek(output.offset, SEEK_SET);
 
 	_curExTypeIndex++;
 	return output;
 }
 
-void CSWorldDeluxeArchive::open(Common::SeekableReadStream *stream) {
+void CSWorldDeluxeArchive::open(Common::File *stream) {
 	close();
 	_mhk = stream;
 
@@ -425,7 +442,7 @@ void CSWorldDeluxeArchive::open(Common::SeekableReadStream *stream) {
 
 	uint32 typeTableOffset = _mhk->readUint32LE();
 
-	_mhk->seek(typeTableOffset);
+	_mhk->seek(typeTableOffset, SEEK_SET);
 
 	_typeTable.resource_types = _mhk->readUint16LE();
 	_types = new OldType[_typeTable.resource_types];
@@ -441,7 +458,7 @@ void CSWorldDeluxeArchive::open(Common::SeekableReadStream *stream) {
 		uint32 oldPos = _mhk->pos();
 
 		// Resource Table/File Table
-		_mhk->seek(_types[i].resource_table_offset + typeTableOffset);
+		_mhk->seek(_types[i].resource_table_offset + typeTableOffset, SEEK_SET);
 		_types[i].resTable.resources = _mhk->readUint16LE();
 		_types[i].resTable.entries = new OldType::ResourceTable::Entries[_types[i].resTable.resources];
 
@@ -454,12 +471,12 @@ void CSWorldDeluxeArchive::open(Common::SeekableReadStream *stream) {
 			debug (4, "Entry[%02x]: ID = %04x (%d)\tOffset = %08x, Size = %08x", j, _types[i].resTable.entries[j].id, _types[i].resTable.entries[j].id, _types[i].resTable.entries[j].offset, _types[i].resTable.entries[j].size);
 		}
 
-		_mhk->seek(oldPos);
+		_mhk->seek(oldPos, SEEK_SET);
 		debug (3, "\n");
 	}
 }
 
-MohawkArchive *MohawkArchive::createMohawkArchive(Common::SeekableReadStream *stream) {
+MohawkArchive *MohawkArchive::createMohawkArchive(Common::File *stream) {
 	uint32 headerTag = stream->readUint32BE();
 
 	MohawkArchive *mohawkArchive = 0;
@@ -469,17 +486,17 @@ MohawkArchive *MohawkArchive::createMohawkArchive(Common::SeekableReadStream *st
 		headerTag = stream->readUint32BE();
 		if (headerTag == ID_RSRC)
 			mohawkArchive = new MohawkArchive();
-	} else if (headerTag == 6 || SWAP_BYTES_32(headerTag) == 6) {
+	} else if (headerTag == 6 || SWAP_32(headerTag) == 6) {
 		// Assume the Living Books v1 archive format
 		mohawkArchive = new LivingBooksArchive_v1();
 	} else {
-		headerTag = SWAP_BYTES_32(headerTag);
+		headerTag = SWAP_32(headerTag);
 		// Use a simple heuristic for testing if it's a CSWorld Deluxe file
 		if (headerTag + 2 < stream->size()) {
-			stream->seek(headerTag);
+			stream->seek(headerTag, SEEK_SET);
 			uint16 typeCount = stream->readUint16LE();
 
-			if (typeCount * 6 + stream->pos() < stream->size()) {
+			if (typeCount * 6 + (uint32)stream->pos() < stream->size()) {
 				bool isDeluxeArchive = true;
 
 				for (uint16 i = 0; i < typeCount; i++) {
@@ -496,7 +513,7 @@ MohawkArchive *MohawkArchive::createMohawkArchive(Common::SeekableReadStream *st
 		}
 	}
 
-	stream->seek(0);
+	stream->seek(0, SEEK_SET);
 
 	if (mohawkArchive)
 		mohawkArchive->open(stream);
