@@ -19,9 +19,10 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-#include <string.h>
 #include "extract_prince.h"
+#include "utils.h"
 #include "common/endian.h"
+#include "common/str.h"
 
 struct FileData;
 
@@ -45,8 +46,8 @@ void ExtractPrince::execute() {
 		_outputPath.setFullPath("./");
 	}
 
-	char *pathBuffer = (char *)malloc(17 * sizeof(char));
-	printf("Unpacking The Prince and the Coward text data... \n");
+	char pathBuffer[100];
+	print("Unpacking The Prince and the Coward text data... ");
 
 	std::string databankFullName = mainDir.getFullPath();
 	sprintf(pathBuffer, "all/databank.ptc");
@@ -59,50 +60,41 @@ void ExtractPrince::execute() {
 	// If the databank.ptc file is not found, we try to access the
 	// uncompressed files directly.
 	if (Common::Filename(databankFullName).exists()) {
-		_databank.open(filename, "rb");
-		printf("DATABANK.PTC loaded, processing... \n");
-		byte *fileTable = openDatabank();
+		_databank = new Databank(databankFullName);
+		print("DATABANK.PTC loaded, processing... ");
 		bool mobsDE = false;
-		for (size_t i = 0; i < _items.size(); i++) {
-			if (!scumm_stricmp(_items[i]._name.c_str(), "variatxt.dat")) {
-				exportVariaTxt(loadFile(i));
+
+		exportVariaTxt(_databank->loadFile("variatxt.dat"));
+		exportInvTxt(_databank->loadFile("invtxt.dat"));
+		exportTalkTxt(_databank->loadFile("talktxt.dat"));
+		exportCredits(_databank->loadFile("credits.dat"));
+
+		int index = _databank->getFileIndex("mob01.lst");
+
+		if (index != -1) {
+			// For DE game data
+			mobsDE = true;
+			_outputPath.setFullName("mob.txt");
+			_fFiles.open(_outputPath, "w");
+			if (!_fFiles.isOpen()) {
+				error("Unable to create mob.txt");
 			}
-			if (!scumm_stricmp(_items[i]._name.c_str(), "invtxt.dat")) {
-				exportInvTxt(loadFile(i));
-			}
-			if (!scumm_stricmp(_items[i]._name.c_str(), "talktxt.dat")) {
-				exportTalkTxt(loadFile(i));
-			}
-			if (!scumm_stricmp(_items[i]._name.c_str(), "credits.dat")) {
-				exportCredits(loadFile(i));
-			}
-			if (!scumm_stricmp(_items[i]._name.c_str(), "mob01.lst")) {
-				// For DE game data
-				mobsDE = true;
-				_outputPath.setFullName("mob.txt");
-				_fFiles.open(_outputPath, "w");
-				if (!_fFiles.isOpen()) {
-					error("Unable to create mob.txt");
+			_fFiles.print("mob.lst\nmob_name - exam text\n");
+			int loc = 0;
+			for (int j = 1; j <= kNumberOfLocations; j++) {
+				_fFiles.print("%d.\n", j);
+				// no databanks for loc 44 and 45
+				if (j != 44 && j != 45) {
+					exportMobs(_databank->loadFile(index + loc));
+					loc++;
 				}
-				_fFiles.print("mob.lst\nmob_name - exam text\n");
-				int loc = 0;
-				for (int j = 1; j <= kNumberOfLocations; j++) {
-					_fFiles.print("%d.\n", j);
-					// no databanks for loc 44 and 45
-					if (j != 44 && j != 45) {
-						exportMobs(loadFile(i + loc));
-						loc++;
-					}
-				}
-				printf("mob.txt - done\n");
-				printf("All done!\n");
-				free(pathBuffer);
-				_fFiles.close();
 			}
+			print("mob.txt - done");
+			print("All done!");
+			_fFiles.close();
 		}
-		free(fileTable);
-		_databank.close();
-		_items.clear();
+
+		delete _databank;
 
 		// For PL game data
 		if (!mobsDE) {
@@ -120,40 +112,33 @@ void ExtractPrince::execute() {
 					sprintf(pathBuffer, "%02d/databank.ptc", loc);
 					databankFullName += pathBuffer;
 					filename = Common::Filename(databankFullName);
-					_databank.open(filename, "rb");
-					if (!_databank.isOpen()) {
+					_databank = new Databank(databankFullName);
+					if (!_databank->isOpen()) {
 						_fFiles.close();
 						error("Unable to open %02d/databank.ptc", loc);
 					}
-					fileTable = openDatabank();
-					for (size_t i = 0; i < _items.size(); i++) {
-						if (!scumm_stricmp(_items[i]._name.c_str(), "mob.lst")) {
-							exportMobs(loadFile(i));
-						}
-					}
-				free(fileTable);
-				_databank.close();
-				_items.clear();
+					exportMobs(_databank->loadFile("mob.lst"));
+
+					delete _databank;
+				}
 			}
-		}
-		printf("mob.txt - done\n");
-		printf("All done!\n");
-		free(pathBuffer);
-		_fFiles.close();
+			print("mob.txt - done");
+			print("All done!");
+			_fFiles.close();
 		}
 	}
 	if (Common::Filename(databankFullName).exists() == false) {
-		printf("Unable to load DATABANK.PTC. Trying to access the uncompressed data files... \n");
+		print("Unable to load DATABANK.PTC. Trying to access the uncompressed data files...");
 
 		// Since we found out that there's no databank.ptc file, we now
 		// try to extract the data from the uncompressed datafiles.
-		printf("Processing variatxt.dat...\n");
+		print("Processing variatxt.dat...");
 		exportVariaTxt(loadFile(mainDir.getFullPath() + "variatxt.dat"));
-		printf("Processing invtxt.dat...\n");
+		print("Processing invtxt.dat...");
 		exportInvTxt(loadFile(mainDir.getFullPath() + "invtxt.dat"));
-		printf("Processing talktxt.dat...\n");
+		print("Processing talktxt.dat...");
 		exportTalkTxt(loadFile(mainDir.getFullPath() + "talktxt.dat"));
-		printf("Processing credits.dat...\n");
+		print("Processing credits.dat...");
 		exportCredits(loadFile(mainDir.getFullPath() + "credits.dat"));
 
 		// Process the mob*.lst files. We have up to 61 files, so we process them recursively.
@@ -162,95 +147,25 @@ void ExtractPrince::execute() {
 		_fFiles.print("mob.lst\nmob_name - exam text\n");
 
 		int mobFileNumber;
-		for (mobFileNumber = 1; mobFileNumber <= 9; ++mobFileNumber) {
-			if (Common::Filename(mainDir.getFullPath() + "mob0" + std::to_string(mobFileNumber) + ".lst").exists() == true) {
+		for (mobFileNumber = 1; mobFileNumber <= 61; ++mobFileNumber) {
+			snprintf(pathBuffer, 100, "mob%02d.lst", mobFileNumber);
+			if (Common::Filename(mainDir.getFullPath() + pathBuffer).exists() == true) {
 				_fFiles.print("%d.\n", mobFileNumber);
-				exportMobs(loadFile(mainDir.getFullPath() + "mob0" + std::to_string(mobFileNumber) + ".lst"));
-				print("Processing mob0%d.lst", mobFileNumber);
-			}
-		}
-		for (mobFileNumber = 10; mobFileNumber <= 61; ++mobFileNumber) {
-			if (Common::Filename(mainDir.getFullPath() + "mob" + std::to_string(mobFileNumber) + ".lst").exists() == true) {
-				_fFiles.print("%d.\n", mobFileNumber);
-				exportMobs(loadFile(mainDir.getFullPath() + "mob" + std::to_string(mobFileNumber) + ".lst"));
-				print("Processing mob%d.lst...", mobFileNumber);
+				exportMobs(loadFile(mainDir.getFullPath() + pathBuffer));
+				print("Processing mob%02d.lst", mobFileNumber);
 			}
 		}
 		_fFiles.close();
-		printf("mob.txt - done\n");
-		printf("All done!\n");
+		print("mob.txt - done");
+		print("All done!");
 	}
 }
 InspectionMatch ExtractPrince::inspectInput(const Common::Filename &filename) {
 	return IMATCH_PERFECT;
 }
 
-byte *ExtractPrince::openDatabank() {
-	_databank.readUint32LE(); // magic
-	uint32 fileTableOffset = _databank.readUint32LE() ^ 0x4D4F4B2D; // MOK-
-	uint32 fileTableSize = _databank.readUint32LE() ^ 0x534F4654; // SOFT
-
-	_databank.seek(fileTableOffset, SEEK_SET);
-
-	byte *fileTable = (byte *)malloc(fileTableSize);
-	byte *fileTableEnd = fileTable + fileTableSize;
-	_databank.read_throwsOnError(fileTable, fileTableSize);
-
-	decrypt(fileTable, fileTableSize);
-
-	for (byte *fileItem = fileTable; fileItem < fileTableEnd; fileItem += 32) {
-		FileEntry item;
-		item._name = (const char *)fileItem;
-		item._offset = READ_LE_UINT32(fileItem + 24);
-		item._size = READ_LE_UINT32(fileItem + 28);
-		_items.push_back(item);
-	}
-	return fileTable;
-}
-
-void ExtractPrince::decrypt(byte *buffer, uint32 size) {
-	uint32 key = 0xDEADF00D;
-	while (size--) {
-		*buffer++ += key & 0xFF;
-		key ^= 0x2E84299A;
-		key += 0x424C4148;
-		key = ((key & 1) << 31) | (key >> 1);
-	}
-}
-
-// DATABANK loader
-ExtractPrince::FileData ExtractPrince::loadFile(int itemIndex) {
-	FileData fileData;
-	fileData._fileTable = 0;
-	fileData._size = 0;
-
-	const FileEntry &entryHeader = _items[itemIndex];
-
-	if (entryHeader._size < 4) {
-		return fileData;
-	}
-
-	fileData._size = entryHeader._size;
-
-	_databank.seek(entryHeader._offset, SEEK_SET);
-
-	fileData._fileTable = (byte *)malloc(fileData._size);
-	_databank.read_throwsOnError(fileData._fileTable, fileData._size);
-
-	if (READ_BE_UINT32(fileData._fileTable) == 0x4D41534D) {
-		Decompressor dec;
-		uint32 decompLen = READ_BE_UINT32(fileData._fileTable + 14);
-		byte *decompData = (byte *)malloc(decompLen);
-		dec.decompress(fileData._fileTable + 18, decompData, decompLen);
-		free(fileData._fileTable);
-		fileData._size = decompLen;
-		fileData._fileTable = decompData;
-	}
-
-	return fileData;
-}
 // Uncompressed datafile loader
-ExtractPrince::FileData ExtractPrince::loadFile(std::string fileName) {
+FileData ExtractPrince::loadFile(const std::string &fileName) {
 	Common::File file;
 	file.open(fileName, "rb");
 	if (!file.isOpen()) {
@@ -362,7 +277,7 @@ void ExtractPrince::exportVariaTxt(FileData fileData) {
 		}
 		free(fileData._fileTable);
 		_fFiles.close();
-		printf("variatxt.txt - done\n");
+		print("variatxt.txt - done");
 	}
 }
 
@@ -406,7 +321,7 @@ void ExtractPrince::exportInvTxt(FileData fileData) {
 		}
 		free(fileData._fileTable);
 		_fFiles.close();
-		printf("invtxt.txt - done\n");
+		print("invtxt.txt - done");
 	}
 }
 
@@ -439,7 +354,7 @@ void ExtractPrince::exportCredits(FileData fileData) {
 		}
 		free(fileData._fileTable);
 		_fFiles.close();
-		printf("credits.txt - done\n");
+		print("credits.txt - done");
 	}
 }
 
@@ -483,7 +398,7 @@ void ExtractPrince::exportTalkTxt(FileData fileData) {
 		}
 		free(fileData._fileTable);
 		_fFiles.close();
-		printf("talktxt.txt - done\n");
+		print("talktxt.txt - done");
 
 		// Additional id data for texts
 		_outputPath.setFullName("talktxt_ids.txt");
@@ -496,7 +411,7 @@ void ExtractPrince::exportTalkTxt(FileData fileData) {
 			_fFiles.print("%d\n", setStringIdArray[i]);
 		}
 		_fFiles.close();
-		printf("talktxt_ids.txt - done\n");
+		print("talktxt_ids.txt - done");
 	}
 }
 
@@ -698,149 +613,6 @@ char ExtractPrince::correctPolishLetter(char c) {
 	}
 }
 
-// John_Doe's implementation
-static const uint16 table1[] = {
-	0x8000, 0x0002,
-	0x4000, 0x0004,
-	0x2000, 0x0008,
-	0x1000, 0x0010,
-	0x0800, 0x0020,
-	0x0400, 0x0040,
-	0x0200, 0x0080,
-	0x0100, 0x0100,
-	0x0080, 0x0200,
-	0x0040, 0x0400
-};
-
-static const uint32 table2[] = {
-	0x0000F000,
-	0x0020FC00,
-	0x00A0FF00,
-	0x02A0FF80,
-	0x06A0FFC0,
-	0x0EA0FFE0,
-	0x1EA0FFF0,
-	0x3EA0FFF8
-};
-
-static const uint16 table3[] = {
-	0x8000, 0x0000,
-	0x4000, 0x0002,
-	0x2000, 0x0006,
-	0x1000, 0x000E,
-	0x0800, 0x001E,
-	0x0400, 0x003E,
-	0x0200, 0x007E,
-	0x0100, 0x00FE,
-	0x0080, 0x01FE,
-	0x0040, 0x03FE,
-	0x0020, 0x07FE,
-	0x0010, 0x0FFE,
-	0x0008, 0x1FFE,
-	0x0004, 0x3FFE,
-	0x0002, 0x7FFE,
-	0x0001, 0xFFFE
-};
-
-void Decompressor::decompress(byte *source, byte *dest, uint32 destSize) {
-	byte *destEnd = dest + destSize;
-	int more;
-	_src = source;
-	_dst = dest;
-	_bitBuffer = 0x80;
-	while (_dst < destEnd) {
-		uint32 ebp;
-		uint16 offset, length;
-		if (getBit()) {
-			if (getBit()) {
-				if (getBit()) {
-					if (getBit()) {
-						if (getBit()) {
-							if (getBit()) {
-								uint32 tableIndex = 0;
-								while (getBit())
-									tableIndex++;
-								length = table3[tableIndex * 2 + 0];
-								do {
-									more = !(length & 0x8000);
-									length = (length << 1) | getBit();
-								} while (more);
-								length += table3[tableIndex * 2 + 1];
-								length++;
-								memcpy(_dst, _src, length);
-								_src += length;
-								_dst += length;
-							}
-							*_dst++ = *_src++;
-						}
-						*_dst++ = *_src++;
-					}
-					*_dst++ = *_src++;
-				}
-				*_dst++ = *_src++;
-			}
-			*_dst++ = *_src++;
-		}
-		if (!getBit()) {
-			if (getBit()) {
-				uint32 tableIndex = getBit();
-				tableIndex = (tableIndex << 1) | getBit();
-				tableIndex = (tableIndex << 1) | getBit();
-				ebp = table2[tableIndex];
-				length = 1;
-			} else {
-				ebp = 0x0000FF00;
-				length = 0;
-			}
-		} else {
-			uint32 tableIndex = 0;
-			while (getBit())
-				tableIndex++;
-			length = table1[tableIndex * 2 + 0];
-			do {
-				more = !(length & 0x8000);
-				length = (length << 1) | getBit();
-			} while (more);
-			length += table1[tableIndex * 2 + 1];
-			tableIndex = getBit();
-			tableIndex = (tableIndex << 1) | getBit();
-			tableIndex = (tableIndex << 1) | getBit();
-			ebp = table2[tableIndex];
-		}
-		offset = ebp & 0xFFFF;
-		do {
-			if (_bitBuffer == 0x80) {
-				if (offset >= 0xFF00) {
-					offset = (offset << 8) | *_src++;
-				}
-			}
-			more = offset & 0x8000;
-			offset = (offset << 1) | getBit();
-		} while (more);
-		offset += (ebp >> 16);
-		length += 2;
-		while (length--) {
-			if (_dst >= destEnd) {
-				return;
-			}
-			*_dst = *(_dst - offset);
-			_dst++;
-		}
-	}
-}
-
-int Decompressor::getBit() {
-	int bit = (_bitBuffer & 0x80) >> 7;
-	_bitBuffer <<= 1;
-	if (_bitBuffer == 0) {
-		_bitBuffer = *_src++;
-		bit = (_bitBuffer & 0x80) >> 7;
-		_bitBuffer <<= 1;
-		_bitBuffer |= 1;
-	}
-	return bit;
-}
-
 #ifdef STANDALONE_MAIN
 int main(int argc, char *argv[]) {
 	ExtractPrince prince(argv[0]);
